@@ -44,6 +44,7 @@ namespace Qt3DExtras
   class QSkyboxEntity;
 }
 
+class Qgs3DAxis;
 class QgsAbstract3DEngine;
 class QgsAbstract3DRenderer;
 class QgsMapLayer;
@@ -71,7 +72,7 @@ class _3D_EXPORT Qgs3DMapScene : public Qt3DCore::QEntity
     Q_OBJECT
   public:
     //! Constructs a 3D scene based on map settings and Qt 3D renderer configuration
-    Qgs3DMapScene( const Qgs3DMapSettings &map, QgsAbstract3DEngine *engine );
+    Qgs3DMapScene( Qgs3DMapSettings &map, QgsAbstract3DEngine *engine );
 
     //! Returns camera controller
     QgsCameraController *cameraController() { return mCameraController; }
@@ -81,6 +82,20 @@ class _3D_EXPORT Qgs3DMapScene : public Qt3DCore::QEntity
     //! Resets camera view to show the whole scene (top view)
     void viewZoomFull();
 
+    /**
+     * Resets camera view to show the extent \a extent (top view)
+     *
+     * \since QGIS 3.26
+     */
+    void setViewFrom2DExtent( const QgsRectangle &extent );
+
+    /**
+     * Calculates the 2D extent viewed by the 3D camera as the vertices of the viewed trapezoid
+     *
+     * \since QGIS 3.26
+     */
+    QVector<QgsPointXY> viewFrustum2DExtent();
+
     //! Returns number of pending jobs of the terrain entity
     int terrainPendingJobsCount() const;
 
@@ -121,6 +136,27 @@ class _3D_EXPORT Qgs3DMapScene : public Qt3DCore::QEntity
      */
     QVector<const QgsChunkNode *> getLayerActiveChunkNodes( QgsMapLayer *layer ) SIP_SKIP;
 
+    /**
+     * Returns the scene extent in the map's CRS
+     *
+     * \since QGIS 3.20
+     */
+    QgsRectangle sceneExtent();
+
+    /**
+     * Returns the 3D axis object
+     *
+     * \since QGIS 3.26
+     */
+    Qgs3DAxis *get3DAxis() { return m3DAxis; }
+
+    /**
+     * Returns the abstract 3D engine
+     *
+     * \since QGIS 3.26
+     */
+    QgsAbstract3DEngine *engine() { return mEngine; }
+
   signals:
     //! Emitted when the current terrain entity is replaced by a new one
     void terrainEntityChanged();
@@ -140,6 +176,13 @@ class _3D_EXPORT Qgs3DMapScene : public Qt3DCore::QEntity
     //! Emitted when the FPS counter is activated or deactivated
     void fpsCounterEnabledChanged( bool fpsCounterEnabled );
 
+    /**
+     * Emitted when the viewed 2D extent seen by the 3D camera has changed
+     *
+     * \since QGIS 3.26
+     */
+    void viewed2DExtentFrom3DChanged( QVector<QgsPointXY> extent );
+
   public slots:
     //! Updates the temporale entities
     void updateTemporal();
@@ -162,20 +205,26 @@ class _3D_EXPORT Qgs3DMapScene : public Qt3DCore::QEntity
     void onDebugShadowMapSettingsChanged();
     void onDebugDepthMapSettingsChanged();
     void onCameraMovementSpeedChanged();
+    void onCameraNavigationModeChanged();
+    void onDebugOverlayEnabledChanged();
+
+    void on3DAxisSettingsChanged();
+
+    bool updateCameraNearFarPlanes();
 
   private:
     void addLayerEntity( QgsMapLayer *layer );
     void removeLayerEntity( QgsMapLayer *layer );
     void addCameraViewCenterEntity( Qt3DRender::QCamera *camera );
+    void addCameraRotationCenterEntity( QgsCameraController *controller );
     void setSceneState( SceneState state );
     void updateSceneState();
     void updateScene();
-    bool updateCameraNearFarPlanes();
     void finalizeNewEntity( Qt3DCore::QEntity *newEntity );
     int maximumTextureSize() const;
 
   private:
-    const Qgs3DMapSettings &mMap;
+    Qgs3DMapSettings &mMap;
     QgsAbstract3DEngine *mEngine = nullptr;
     //! Provides a way to have a synchronous function executed each frame
     Qt3DLogic::QFrameAction *mFrameAction = nullptr;
@@ -193,10 +242,14 @@ class _3D_EXPORT Qgs3DMapScene : public Qt3DCore::QEntity
     QList<Qgs3DMapScenePickHandler *> mPickHandlers;
     //! List of lights in the scene
     QList<Qt3DCore::QEntity *> mLightEntities;
-    //! List of light origins in the scene
-    QList<Qt3DCore::QEntity *> mLightOriginEntities;
     QList<QgsMapLayer *> mModelVectorLayers;
     QgsSkyboxEntity *mSkybox = nullptr;
+    //! Entity that shows rotation center = useful for debugging camera issues
+    Qt3DCore::QEntity *mEntityRotationCenter = nullptr;
+
+    //! 3d axis visualization
+    Qgs3DAxis *m3DAxis = nullptr;
+
 };
 
 #endif // QGS3DMAPSCENE_H
